import unittest
import subprocess
import requests
import time
import json

BASE_URL = 'http://127.0.0.1:29410'


class TestServer(unittest.TestCase):
    _server_process = None

    @staticmethod
    def get_current_dict() -> dict:
        target_url = BASE_URL + '/res'
        resp = requests.get(target_url)
        resp_dict = json.loads(resp.text)
        return resp_dict

    def get_res_status(self, name) -> str:
        current_dict = self.get_current_dict()
        return current_dict[str(name)]['status']

    @classmethod
    def setUpClass(cls):
        super(TestServer, cls).setUpClass()
        cls._server_process = subprocess.Popen('glockrs start', shell=True)
        time.sleep(3)

    def test_1_add(self):
        target_url = BASE_URL + '/res/add'

        name_list = list(range(0, 30))
        for label in range(5):
            for _ in range(5):
                # 'data' seems does not work?
                each_url = target_url + '?name={}&label={}'.format(name_list.pop(), label)
                resp = requests.post(each_url)
                assert resp.ok

    def test_2_show_all(self):
        current_dict = self.get_current_dict()
        assert len(current_dict) == 25

    def test_3_acquire_name(self):
        acquire_url = BASE_URL + '/res/acquire'
        release_url = BASE_URL + '/res/release'

        # check 15-19
        for each_name in range(15, 20):
            # check its status
            assert self.get_res_status(each_name) == 'FREE'

            # acquire
            each_acquire_url = acquire_url + '?name={}'.format(each_name)
            resp = requests.post(each_acquire_url)
            assert resp.ok

            # check its status
            assert self.get_res_status(each_name) == 'BUSY'

            # try to acquire it again
            resp = requests.post(each_acquire_url)
            resp_dict = json.loads(resp.text)
            assert not resp_dict['result']

            # try to acquire by label
            each_acquire_label_url = acquire_url + '?label={}'.format(2)
            resp = requests.post(each_acquire_label_url)
            resp_dict = json.loads(resp.text)
            assert not resp_dict['result']

            # release
            each_release_url = release_url + '?name={}'.format(each_name)
            resp = requests.post(each_release_url)
            assert resp.ok

            # check its status
            assert self.get_res_status(each_name) == 'FREE'

    def test_4_acquire_label(self):
        acquire_url = BASE_URL + '/res/acquire'
        release_url = BASE_URL + '/res/release'

        # check 15-19
        for each_name in range(15, 20):
            each_status = self.get_res_status(each_name)
            assert each_status == 'FREE'

        # acquire label '2'
        each_release_label_url = acquire_url + '?label={}'.format(2)
        resp = requests.post(each_release_label_url)
        resp_dict = json.loads(resp.text)
        assert resp_dict['result']

        # check 15-19
        for each_name in range(15, 20):
            each_status = self.get_res_status(each_name)
            assert each_status == 'BUSY'

        # release label '2'
        each_release_label_url = release_url + '?label={}'.format(2)
        resp = requests.post(each_release_label_url)
        resp_dict = json.loads(resp.text)
        assert resp_dict['result']

        # check 15-19
        for each_name in range(15, 20):
            each_status = self.get_res_status(each_name)
            assert each_status == 'FREE'

    @classmethod
    def tearDownClass(cls):
        super(TestServer, cls).tearDownClass()
        cls._server_process.kill()


if __name__ == '__main__':
    unittest.main()



from fastapi import FastAPI
import uvicorn
import fire

from glockr.manager import GResourceManager

# --- API below ---

# TODO API for updating resource?

app = FastAPI()


@app.get("/")
def hello():
    return {"hello": "from glockr server"}


@app.get("/res")
def res():
    return GResourceManager.show_all()


@app.post("/res/add")
def add(name: str, label: str = None):
    return GResourceManager.add(name, label).to_dict()


@app.post("/res/remove")
def remove(name: str):
    return GResourceManager.remove(name).to_dict()


@app.post("/res/acquire")
def acquire(name: str = None, label: str = None):
    """ acquire resource """
    if name:
        return GResourceManager.acquire_res(name).to_dict()
    if label:
        return GResourceManager.acquire_label(label).to_dict()
    return {
        'error': 'at least fill name or label'
    }


@app.post("/res/release")
def release(name: str = None, label: str = None):
    """ release resource """
    if name:
        return GResourceManager.release_res(name).to_dict()
    if label:
        return GResourceManager.release_label(label).to_dict()
    return {
        'error': 'at least fill name or label'
    }


def start_server(port: int = None):
    # TODO how to sync this change to client ??
    if not port:
        port = 29410

    uvicorn.run(
        app,
        host="0.0.0.0",
        port=port,
        log_level="info"
    )


def main():
    fire.Fire({
        'start': start_server,
    })


if __name__ == '__main__':
    main()



class GResourceStatus(object):
    FREE = 'FREE'
    BUSY = 'BUSY'


class GResource(object):
    def __init__(self, name, label=None, **kwargs):
        self.name = name
        self.label = label
        self.status = GResourceStatus.FREE

        self.__dict__.update(kwargs)

    def available(self) -> bool:
        return self.status == GResourceStatus.FREE

    def acquire(self):
        if self.available():
            self.status = GResourceStatus.BUSY
            return GResourceResponse(True)
        return GResourceResponse(False, 'res {} status: {}'.format(self.name, self.status))

    def release(self):
        # just make sure, res becomes free.
        self.status = GResourceStatus.FREE
        return GResourceResponse(True)


class GResourceResponse(object):
    def __init__(self, result: bool, reason: str = None):
        self.result = result
        self.reason = reason if reason is not None else ''

    def to_dict(self):
        return {
            'result': self.result,
            'reason': self.reason,
        }


class GResourceManager(object):
    _store = dict()

    @classmethod
    def add(cls, res_name: str, res_label: str = None, **kwargs) -> GResourceResponse:
        if cls.is_existed(res_name):
            return GResourceResponse(False, 'res {} already existed'.format(res_name))

        res = GResource(res_name, label=res_label, **kwargs)
        cls._store[res_name] = res
        return GResourceResponse(True)

    @classmethod
    def remove(cls, res_name: str):
        if not cls.is_existed(res_name):
            return GResourceResponse(False, 'res {} not existed'.format(res_name))

        del cls._store[res_name]

    @classmethod
    def acquire_res(cls, res_name: str) -> GResourceResponse:
        if not cls.is_existed(res_name):
            return GResourceResponse(False, 'res {} not existed'.format(res_name))

        return cls._store[res_name].acquire()

    @classmethod
    def acquire_label(cls, label_name: str) -> GResourceResponse:
        target_list = list()
        for each_name, each_res in cls._store.items():
            if each_res.label == label_name:
                target_list.append(each_res)

        # no data found?
        if not target_list:
            return GResourceResponse(False, 'label {} not found'.format(label_name))

        # make sure they can be acquired before acquirement
        for each_res in target_list:
            if not each_res.available():
                return GResourceResponse(False, 'res {} not available'.format(each_res.name))

        for each_res in target_list:
            acquire_result = cls.acquire_res(each_res.name)
            if not acquire_result.result:
                return acquire_result

        return GResourceResponse(True)

    @classmethod
    def release_res(cls, res_name):
        if not cls.is_existed(res_name):
            return GResourceResponse(False, 'res {} not existed'.format(res_name))

        return cls._store[res_name].release()

    @classmethod
    def release_label(cls, label_name):
        target_list = list()
        for each_name, each_res in cls._store.items():
            if each_res.label == label_name:
                target_list.append(each_res)

        for each_res in target_list:
            release_result = cls.release_res(each_res.name)
            if not release_result.result:
                return release_result

        return GResourceResponse(True)

    @classmethod
    def show_all(cls):
        return {
            each_name: each_res.__dict__
            for each_name, each_res in cls._store.items()
        }

    @classmethod
    def is_existed(cls, res_name: str) -> bool:
        return res_name in cls._store



import unittest
import subprocess
import requests
import time
import json

BASE_URL = 'http://127.0.0.1:29410'


class TestClient(unittest.TestCase):
    _server_process = None

    @staticmethod
    def get_current_dict() -> dict:
        target_url = BASE_URL + '/res'
        resp = requests.get(target_url)
        resp_dict = json.loads(resp.text)
        return resp_dict

    def get_res_status(self, name) -> str:
        current_dict = self.get_current_dict()
        return current_dict[str(name)]['status']

    @classmethod
    def setUpClass(cls):
        super(TestClient, cls).setUpClass()
        cls._server_process = subprocess.Popen('glockrs', shell=True)
        time.sleep(3)

    def test_1_add(self):
        name_list = list(range(0, 30))
        for label in range(5):
            for _ in range(5):
                cmd = 'glockrc add {} {}'.format(name_list.pop(), label)
                subprocess.check_call(cmd, shell=True)

    def test_2_show_all(self):
        current_dict = self.get_current_dict()
        assert len(current_dict) == 25

    @classmethod
    def tearDownClass(cls):
        super(TestClient, cls).tearDownClass()
        cls._server_process.kill()


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages


setup(
    name='glockr',
    version='0.1.3',
    description='global lockable resources for all',
    author='williamfzc',
    author_email='fengzc@vip.qq.com',
    url='https://github.com/williamfzc/glockr',
    packages=find_packages(),
    install_requires=[
        'fastapi',
        'requests',
        'uvicorn',
        'fire',
    ],
    entry_points={
        "console_scripts": [
            "glockrs = glockr.server:main",
            "glockrc = glockr.client:main",
        ],
    },
)



import requests
import fire
import requests.exceptions


# TODO API for waiting until release


class GClient(object):
    _PORT = '29410'
    _BASE_URL = 'http://127.0.0.1:{}'.format(_PORT)
    _URL_DICT = {
        'heart': _BASE_URL + '/',
        'add': _BASE_URL + '/res/add',
        'remove': _BASE_URL + '/res/remove',
        'show_all': _BASE_URL + '/res',

        'acquire': {
            'name': _BASE_URL + '/res/acquire?name={}',
            'label': _BASE_URL + '/res/acquire?label={}',
        },
        'release': {
            'name': _BASE_URL + '/res/release?name={}',
            'label': _BASE_URL + '/res/release?label={}',
        }
    }

    @classmethod
    def heartbeat(cls) -> bool:
        target_url = cls._URL_DICT['heart']
        try:
            resp = requests.get(target_url)
        except requests.exceptions.ConnectionError:
            return False
        return resp.ok

    @classmethod
    def acquire_name(cls, name: str):
        print(cls._acquire('name', name))

    @classmethod
    def acquire_label(cls, label: str):
        print(cls._acquire('label', label))

    @classmethod
    def release_name(cls, name: str):
        print(cls._release('name', name))

    @classmethod
    def release_label(cls, label: str):
        print(cls._release('label', label))

    @classmethod
    def add(cls, name: str, label: str = None):
        target_url = cls._URL_DICT['add'] + '?name={}'.format(name)
        if label:
            target_url += '&label={}'.format(label)
        resp = requests.post(target_url)
        print(resp.json())

    @classmethod
    def remove(cls, name: str):
        target_url = cls._URL_DICT['remove'] + '?name={}'.format(name)
        resp = requests.post(target_url)
        print(resp.json())

    @classmethod
    def show_all(cls):
        target_url = cls._URL_DICT['show_all']
        resp = requests.get(target_url)
        print(resp.json())

    @classmethod
    def _acquire(cls, acquire_type: str, content: str):
        target_url = cls._URL_DICT['acquire'][acquire_type].format(content)
        resp = requests.post(target_url)
        return resp.json()

    @classmethod
    def _release(cls, release_type: str, content: str):
        target_url = cls._URL_DICT['release'][release_type].format(content)
        resp = requests.post(target_url)
        return resp.json()


def main():
    fire.Fire(GClient)


if __name__ == '__main__':
    main()
